/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <UpTools/UpResolverSimple.h>
#include <UpTools/UpOp.h>


bool dnsbl(const std::string& dnsblServerDomain, const UpAddress& addrToCheck) {
	if( addrToCheck.family()!=AF_INET ) return false; // only for IPv4 addresses!
	unsigned char* a( (unsigned char*)addrToCheck.ptr() );
	std::string searchDomain;
	searchDomain<<a[3]<<'.'<<a[2]<<'.'<<a[1]<<'.'<<a[0]<<'.'<<dnsblServerDomain;
	// would use ns_t_any to check also txt records
	UpDnsQuerySimple q(searchDomain,ns_t_a);
	UpResolverSimple res;
	res.query(q);
	// if( q.getCount!=0 ) std::cout<<q<<endl;
	return q.getCount()==0;
}

bool dnsbl(const std::string& dnsblServerDomain, const std::string& addrToCheck) {
	UpAddress addr(addrToCheck);
	return dnsbl(dnsblServerDomain,addr);
}

bool dnsbl(const std::string& dnsblServerDomain, const char* addrToCheck) {
	UpAddress addr(addrToCheck);
	return dnsbl(dnsblServerDomain,addr);
}



#include <iostream>

using namespace std;


void usage(const char* progname) {
	cerr<<"Usage: "<<progname<<" rblMaintainerDomain IPv4Address"<<endl;
	exit(1);
}

	
int main(int argc, char *argv[]) {
	if( argc!=3 ) usage(argv[0]);
	cout << ( dnsbl(argv[1],argv[2]) ? "good" : "bad" ) << endl;
	return 0;
}

